from google.oauth2 import service_account #Service Account
import pandas_gbq #Pandas GBQ.
from google.cloud import bigquery #To connect to BigQuery.

def read_big_query(query, project, creds_path):
    '''
    query (str): SQL query 
    project (str): Then name of the Big Query project
    creds_path (str): Path to service account json file (_eg. 'creds/google_creds.json'_)

    returns: Pandas dataframe with query data. 
    '''

    #Authenticate
    credentials = service_account.Credentials.from_service_account_file(creds_path)
    pandas_gbq.context.credentials = credentials

    #Read in dataframe.
    data_frame = pandas_gbq.read_gbq(
        query,
        project_id=project,
        dialect='standard')

    return data_frame

#Write DF to Big Query
def df_to_big_query(df, proj, ds, tb, creds_path, schema=None, overwrite=True):
    '''
    df (pandas dataframe): The data that you want uploaded to Big Query.
    proj (str): The name of the Big Query project.
    ds (str): The name of the Big Query dataset.
    tb (str): The name of the Big Query table.
    creds_path (str): Path to the service account json file (_eg. 'creds/google_creds.json'_)
    schema (dict): Schema of the Big Query dataset. Default is None. (UNTESTED)
    overwrite (bool):  True overwrites the existing table, False appends data to the end of the table.

    returns: Nothing (Might want to change this). 
    '''

    #Authenticate Big Query.
    client = bigquery.Client.from_service_account_json(creds_path, project=proj)

    #Connect to client.
    dataset = client.dataset(ds)
    table_id = proj + "." + ds + "." + tb
    table = dataset.table(tb)

    if overwrite:
        #Delete the table.
        client.delete_table(table_id)
        print("Deleted table " + table_id)

        #Create the table.
        if schema:
            table_ref = dataset.table('my_table')
            table = bigquery.Table(table_ref, schema=schema)
            table = client.create_table(table)  # API request
            table = bigquery.table.Table(table_id, schema)
        else:
            client.create_table(table_id)
        print("Created table " + table_id)

    #Load data to the table.
    client.load_table_from_dataframe(df, table).result()
    print("Uploaded DF to " + table_id)


from pymongo import MongoClient
import gridfs
from bson.objectid import ObjectId

def get_mongo_client(mongo_string):
    client = MongoClient(mongo_string)
    return client

def create_docs(client, database, collection, data, many=False):
    # Get db and collection.
    db = client[database]
    col = db[collection]

    if many:
        # Insert data.
        doc = col.insert_many(data)

        # Return doc id
        return doc.inserted_ids

    else:
        # Insert data.
        doc = col.insert_one(data)

        # Return doc id
        return doc.inserted_id


def get_docs(client, database, collection, query, many=False):
    # Get db and collectino
    db = client[database]
    col = db[collection]

    if many:
        # Find
        data = col.find(query)

        return [row for row in data]
    else:
        # Find
        data = col.find_one(query)

        return data


def update_docs(client, database, collection, query, data, upsert=False):
    # Get db and collectino
    db = client[database]
    col = db[collection]

    # Update one.
    col.update_one(query, data, upsert=upsert)


def delete_docs(client, database, collection, query):
    # Get db and collectino
    db = client[database]
    col = db[collection]

    # Update one.
    col.delete_one(query)

def get_file(client, database, file_id):
    fs = gridfs.GridFS(database=client[database])
    file_data = fs.find_one({"_id": ObjectId(file_id)})
    data = file_data.read()
    return data

def upload_file(client, database, file):
    fs = gridfs.GridFS(database=client[database])
    file_id = fs.put(file, file_name=str(file), content_type=str(file.content_type))
    return file_id



#Import packages. 
from googleapiclient.discovery import build
from google_auth_oauthlib.flow import InstalledAppFlow
from google.auth.transport.requests import Request
import base64
import pickle
import os
import email
import datetime as dt
from bs4 import BeautifulSoup
import pandas as pd

from .general import get_google_service

def mark_as_read(service, message_id):

    #Modify message label.
    response = service.users().messages().modify(userId="me", id=message_id, body= {
        "removeLabelIds" : ["UNREAD"]
    }).execute()

    return response

def list_messages(service, max_results=1000, labels=None, query=None, read=False):

    #List all the messages. 
    messages = service.users().messages().list(userId="me",labelIds=labels, maxResults=max_results, q=query).execute()

    return messages.get("messages")

def get_attachements(service, message_id, storage_path):

    message = service.users().messages().get(userId="me", id=message_id).execute()

    for part in message['payload']['parts']:
      if part['filename']:

        file_data = base64.urlsafe_b64decode(part['body']['data'].encode('UTF-8'))

        path = ''.join([storage_path, part['filename']])

        f = open(path, 'w')
        f.write(file_data)
        f.close()

def parse_email(service, message_id, content_type):
    #Get the message. 
    message = service.users().messages().get(userId="me", id=message_id, format="raw").execute()

    #Decode the Base64 format.
    raw = message.get("raw")
    mail = email.message_from_bytes(base64.urlsafe_b64decode(raw))

    #Get message details.
    date = mail.get("Date").split("-")[0].strip()
    date = dt.datetime.strptime(date, "%a, %d %b %Y %H:%M:%S")
    from_ = mail.get("From")
    subject = mail.get("Subject")
    to_ = mail.get("To")

    #Loop through each part of the message. 
    body = []
    for part in mail.walk():
        #Get only the plain text. 
        if part.get_content_type()==content_type:
            #Add it to the message. 
            body.append(part.as_string())

    output = {
        "date" : date,
        "from_" : from_,
        "subject" : subject,
        "to_" : to_,
        "body" : body
    }

    return output


import datetime
import pandas as pd
import os
import numpy as np
import requests
import base64
from googleapiclient import discovery
from oauth2client.service_account import ServiceAccountCredentials
from google.cloud import bigquery
from dotenv import load_dotenv
import boto3
from ast import literal_eval
load_dotenv()

class clockify():
    def __init__(self):
        self.base_url = os.environ['CLOCKIFY_BASE_URL']
        self.workspace_id = os.environ['CLOCKIFY_WORKSPACE_ID']
        self.headers = {'X-Api-key':os.environ['CLOCKIFY_API_KEY'],'content-type':'application/json'}
        self.yesterday = (datetime.date.today() - datetime.timedelta(days=1)).strftime('%Y-%m-%d')
        self.start = datetime.datetime.strftime(datetime.datetime.today() - datetime.timedelta(days=10),'%Y-%m-%d')
        self.end = datetime.datetime.strftime(datetime.datetime.today() - datetime.timedelta(days=3),'%Y-%m-%d')
        self.s3 = boto3.resource('s3',
                            aws_access_key_id=os.environ['AWS_ACCESS_KEY_ID'],
                            aws_secret_access_key=os.environ['AWS_SECRET_ACCESS_KEY']
                            )

    def build_user_dictionary(self):
        results = requests.get(self.base_url + 'workspaces/' + str(self.workspace_id) + '/users',headers=self.headers)
        users = []
        for user in results.json():
            users.append({'name':user['name'],'id':user['id']})
        for index_i,user in enumerate(users):
            params = {'start':self.start + 'T00:00:00.000Z','end':self.end + 'T23:59:59.999Z'}
            results = requests.get(self.base_url + 'workspaces/' + str(self.workspace_id) + '/user/' + user['id'] + '/time-entries',params=params,headers=self.headers).json()
            projects = []
            project_ids = {}
            for project in results:
                if project['projectId'] in project_ids:
                    pass
                else:
                    project_ids[project['projectId']] = project['projectId']
            for project in project_ids:
                if type(project) != type(None):
                    project_info = {}
                    project_info['id'] = project
                    projects.append(project_info)
            users[index_i]['projects'] = projects
        for user in users:
            for index_i,project in enumerate(user['projects']):
                project_info = requests.get(self.base_url + 'workspaces/' + str(self.workspace_id) + '/projects/' + project['id'],headers=self.headers).json()
                project['name'] = project_info['name']
                tasks_info = requests.get(self.base_url + 'workspaces/' + str(self.workspace_id) + '/projects/' + project['id'] + '/tasks',params=params,headers=self.headers).json()
                task_macro = []
                for task in tasks_info:
                    task_micro = {}
                    task_micro['id'] = task['id']
                    task_micro['name'] = task['name']
                    task_macro.append(task_micro)
                project['tasks'] = task_macro
        return self.s3.Object(os.environ['AWS_BUCKET'], os.environ['AWS_CLOCKIFY_USERS_FILENAME']).put(Body=json.dumps(users))

    def get_time_entries_from_user_dictionary(self):
        obj = self.s3.Object(os.environ['AWS_BUCKET'], os.environ['AWS_CLOCKIFY_USERS_FILENAME'])
        body = obj.get()['Body'].read()
        users = literal_eval(body.decode('utf8'))
        time_entry_macro = []
        for user in users:
            params = {'start':self.start + 'T00:00:00.000Z','end':self.end + 'T23:59:59.999Z'}
            time_entries = requests.get(self.base_url + 'workspaces/' + str(self.workspace_id) + '/user/' + user['id'] + '/time-entries',params=params,headers=self.headers).json()
            for time_entry in time_entries:
                project_id = time_entry['projectId']
                task_id = time_entry['taskId']
                start = datetime.datetime.strptime(time_entry['timeInterval']['start'],'%Y-%m-%dT%H:%M:%SZ')
                end = datetime.datetime.strptime(time_entry['timeInterval']['end'],'%Y-%m-%dT%H:%M:%SZ')
                for project in user['projects']:
                    if project['id'] == project_id:
                        time_entry_micro = {}
                        time_entry_micro['name'] = user['name']
                        time_entry_micro['project'] = project['name']
                        time_entry_micro['start'] = start
                        time_entry_micro['end'] = end
                        time_entry_micro['duration'] = int(end.timestamp() - start.timestamp())
                        time_entry_micro['date'] = start.date()
                        for task in project['tasks']:
                            if task['id'] == task_id:
                                time_entry_micro['task'] = task['name']
                            else:
                                time_entry_micro['task'] = None
                        time_entry_macro.append(time_entry_micro.copy())
        creds_path = os.environ['BIGQUERY_CREDS_PATH']
        bq = bigQuery(os.environ['BIGQUERY_DATASET'],'clockify',creds_path)
        request_error = bq.main(time_entry_macro)
        return time_entry_macro



from __future__ import print_function
import pickle
import os.path
from googleapiclient.discovery import build
from google_auth_oauthlib.flow import InstalledAppFlow
from google.auth.transport.requests import Request
import pandas as pd

# If modifying these scopes, delete the file token.pickle.
SCOPES = ['https://www.googleapis.com/auth/spreadsheets']

def get_google_sheet(spreadsheet_id, sheet_name, range_name):
    """Shows basic usage of the Sheets API.
    Prints values from a sample spreadsheet.
    """

    creds = None
    # The file token.pickle stores the user's access and refresh tokens, and is
    # created automatically when the authorization flow completes for the first
    # time.
    if os.path.exists('config/token.pickle'):
        with open('config/token.pickle', 'rb') as token:
            creds = pickle.load(token)
    # If there are no (valid) credentials available, let the user log in.
    if not creds or not creds.valid:
        if creds and creds.expired and creds.refresh_token:
            creds.refresh(Request())
        else:
            flow = InstalledAppFlow.from_client_secrets_file(
                'config/credentials.json', SCOPES)
            creds = flow.run_local_server(port=0)
        # Save the credentials for the next run
        with open('config/token.pickle', 'wb') as token:
            pickle.dump(creds, token)

    service = build('sheets', 'v4', credentials=creds)

    # Call the Sheets API
    SPREADSHEET_ID = spreadsheet_id
    SHEET_NAME = sheet_name + "!" + range_name

    sheet = service.spreadsheets()
    result = sheet.values().get(spreadsheetId=SPREADSHEET_ID, range=SHEET_NAME).execute()
    values = result.get('values', [])

    return pd.DataFrame(values)

def update_google_sheet(spreadsheet_id,sheet_name,range,dataframe):
    """Shows basic usage of the Sheets API.
    Prints values from a sample spreadsheet.
    """
    creds = None
    # The file token.pickle stores the user's access and refresh tokens, and is
    # created automatically when the authorization flow completes for the first
    # time.
    if os.path.exists('config/token.pickle'):
        with open('config/token.pickle', 'rb') as token:
            creds = pickle.load(token)
    # If there are no (valid) credentials available, let the user log in.
    if not creds or not creds.valid:
        if creds and creds.expired and creds.refresh_token:
            creds.refresh(Request())
        else:
            flow = InstalledAppFlow.from_client_secrets_file(
                'config/credentials.json', SCOPES)
            creds = flow.run_local_server(port=0)
        # Save the credentials for the next run
        with open('config/token.pickle', 'wb') as token:
            pickle.dump(creds, token)
    service = build('sheets', 'v4', credentials=creds)
    values = dataframe.values.tolist()
    body = {
        'values':values
    }
    SPREADSHEET_ID=spreadsheet_id
    RANGE_NAME=sheet_name + '!' + range
    # Call the Sheets API
    sheet = service.spreadsheets()
    request = sheet.values().update(spreadsheetId=SPREADSHEET_ID,
                                range=RANGE_NAME,valueInputOption='USER_ENTERED',body=body)
    response = request.execute()



from distutils.core import setup

setup(
    name = 'fp_utils',
    packages = ['fp_utils', 'fp_utils.big_query', 'fp_utils.mongo', 'fp_utils.google','fp_utils.clockify'],
    version = '1.2.0',  #
    description = 'Most commonly used Fifth Partners functions in PyPi for easy pip install. ',
    author = 'Nathan Duncan',
    author_email = 'nduncan@fifthpartners.com',
    url = 'https://github.com/fifth-partners/fp-utils',
    download_url = 'https://github.com/fifth-partners/fp-utils/archive/1.2.0.tar.gz',
    keywords = ['fifth_partners', 'utils'],
    install_requires=['google-cloud-bigquery', 'google-api-python-client','google-auth-httplib2', 'google-auth-oauthlib',
    'pymongo', 'datetime', 'bs4', 'pathlib','pandas','numpy','requests','oauth2client','python-dotenv','boto3',
    'pybase64','ast'],
    classifiers = [
    'Development Status :: 3 - Alpha',      # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable"
    'Intended Audience :: Developers',
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7'
  ],
)



#Import packages. 
from googleapiclient.discovery import build
from google_auth_oauthlib.flow import InstalledAppFlow
from google.auth.transport.requests import Request
import pickle
import os

def get_google_service(service, creds_path, token_path):
    #Handle scopes. 
    SCOPES = {
        "sheets" : ['https://www.googleapis.com/auth/spreadsheets'], 
        "slides" : ['https://www.googleapis.com/auth/presentations'],
        "drive" : ["https://www.googleapis.com/auth/drive"],
        "gmail" : ["https://www.googleapis.com/auth/gmail.modify"]
    }
    
    # https://developers.google.com/sheets/api/quickstart/python
    creds = None
    # The file token.pickle stores the user's access and refresh tokens, and is
    # created automatically when the authorization flow completes for the first
    # time.
    if os.path.exists(token_path):
        with open(token_path, 'rb') as token:
            creds = pickle.load(token)
    # If there are no (valid) credentials available, let the user log in.
    if not creds or not creds.valid:
        if creds and creds.expired and creds.refresh_token:
            creds.refresh(Request())
        else:
            flow = InstalledAppFlow.from_client_secrets_file(
                creds_path, SCOPES[service])
            creds = flow.run_local_server(port=0)
        # Save the credentials for the next run
        with open(token_path, 'wb') as token:
            pickle.dump(creds, token)
    
    # Different for sheets and slides. 
    if service=="sheets":
      service = build('sheets', 'v4', credentials=creds, cache_discovery=False)
    elif service=="slides":
      service = build('slides', 'v1', credentials=creds, cache_discovery=False)
    elif service=="drive":
      service = build('drive', 'v3', credentials=creds, cache_discovery=False)
    elif service == "gmail":
        service = build('gmail', 'v1', credentials=creds, cache_discovery=False)
    
    #Return service. 
    return service
    